/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef DATEBOOKDAY_H
#define DATEBOOKDAY_H

#include <qvbox.h>
#include <qtable.h>
#include <qlist.h>
#include <organizer.h>

class QResizeEvent;
class QDateTime;
class DateBookDayHeader;
class Event;
class QPaintEvent;
class QMouseEvent;

class DateBookDayItem : public QTableItem
{
public:
    DateBookDayItem( QTable *parent, int t );
};

class DateBookDayView : public QTable
{
    Q_OBJECT

public:
    DateBookDayView( QWidget *parent, const char *name );
};

class DateBookDay;
class DateBookDayWidget : public QWidget
{
    Q_OBJECT

public:
    DateBookDayWidget( const Event &e, DateBookDay *db );

    void setX( int x );
    void setWidth( int w );

    Event event() const { return ev; }

signals:
    void deleteMe( const QDate &date, DateBookDayWidget *me );
    void editMe( const QDate &date, DateBookDayWidget *me );

protected:
    void paintEvent( QPaintEvent *e );
    void mousePressEvent( QMouseEvent *e );

private:
    void place();

private:
    Event ev;
    DateBookDay *dateBook;
    int xpos, wid;
    QString text;

};

class EventList : public QList<DateBookDayWidget> {
public:
    EventList() : QList<DateBookDayWidget>() { }

    int compareItems( QCollection::Item, QCollection::Item );
};

typedef QListIterator<DateBookDayWidget> EventListIterator;

class DateBookDay : public QVBox
{
    Q_OBJECT

public:
    DateBookDay( QWidget *parent, const char *name );
    void selectedDates( QDateTime &start, QDateTime &end );
    QDate date() const;
    DateBookDayView *dayView() const { return view; }
    void saveAllEvents( const QString& fn );

protected:
    void resizeEvent( QResizeEvent* );

public slots:
    void setDate( int y, int m, int d );
    void addEvent( const Event &e );
    void deleteEvent( const Event &e );

signals:
    void newEvent( const Event & );
    void removeEvent( const Event & );
    void editEvent( const Event& );

private slots:
    void dateChanged( int y, int m, int d );
    void deleteEvent( const QDate &date, DateBookDayWidget *me );
    void editEvent( const QDate &date, DateBookDayWidget *me );

private:
    void relayoutPage( EventList *l );

private:
    QDate currDate;
    DateBookDayView *view;
    DateBookDayHeader *header;
    typedef QMap<QString, EventList * > EventMap;
    EventMap events;
};

#endif
