/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef DATEBOOKWEEK
#define DATEBOOKWEEK

#include <qvbox.h>
#include <qhbox.h>
#include <qlistview.h>
#include <qlist.h>

#include <organizer.h>

class QSpinBox;
class QLabel;
class DateBookWeekHeader;
class QResizeEvent;
class QDate;

class DateBookWeekItem : public QListViewItem
{
public:
    DateBookWeekItem( QListView *parent, int t );

    void paintCell( QPainter *, const QColorGroup & cg,
		    int column, int width, int alignment );

};


class DateBookWeekView : public QListView
{
    Q_OBJECT

public:
    DateBookWeekView( QWidget *parent = 0, const char *name = 0 );

signals:
    void showDay( int d );

protected:
    void resizeEvent( QResizeEvent *e );

private slots:
    void dateClicked( QListViewItem *i, const QPoint &p, int c );

};

class DateBookWeek;
class DateBookWeekWidget : public QWidget
{
    Q_OBJECT

public:
    DateBookWeekWidget( const Event &e, DateBookWeek *db, int y, int w );

    void setX( int x );
    void setWidth( int w );

    Event event() const { return ev; }

signals:
    void deleteMe( int y, int w, DateBookWeekWidget *me );

protected:
    void mousePressEvent( QMouseEvent *e );

private:
    void place();

private:
    Event ev;
    DateBookWeek *dateBook;
    int xpos, wid;
    int year, week;

};

class DateBookWeek : public QVBox
{
    Q_OBJECT

public:
    DateBookWeek( QWidget *parent = 0, const char *name = 0 );
    void setDate( int y, int w );

    QDate date() const;
    DateBookWeekView *weekView() const { return view; }

public slots:
    void addEvent( const Event &e );
    void deleteEvent( const Event &e );
    
signals:
    void showDate( const QDate &d );
    void removeEvent( const Event &e );
    void newEvent( const Event &e );
    
private slots:
    void showDay( int day );
    void dateChanged( int y, int w );
    void deleteEvent( int y, int w, DateBookWeekWidget *me );

private:
    void relayoutPage( QList<DateBookWeekWidget> *l );

private:
    DateBookWeekHeader *header;
    DateBookWeekView *view;
    int year, week;
    QMap<QString, QList<DateBookWeekWidget> * > events;

};


#endif
