/* ---------------------------------------------------------------------- */
/*                                                                        */
/* [main.C]                        Konsole                                */
/*                                                                        */
/* ---------------------------------------------------------------------- */
/*                                                                        */
/* Copyright (c) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>        */
/*                                                                        */
/* This file is part of Konsole, an X terminal.                           */
/*                                                                        */
/* The material contained in here more or less directly orginates from    */
/* kvt, which is copyright (c) 1996 by Matthias Ettrich <ettrich@kde.org> */
/*                                                                        */
/* ---------------------------------------------------------------------- */
/*									      */
/* Ported Konsole to Qt/Embedded                                              */
/*									      */
/* Copyright (C) 2000 by John Ryland <jryland@trolltech.com>                  */
/*									      */
/* -------------------------------------------------------------------------- */

#include <qdir.h>
#include <qevent.h>
#include <qdragobject.h>
#include <qobjectlist.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qpushbutton.h>
#include <qfontdialog.h>
#include <qglobal.h>
#include <qpainter.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qaction.h>
#include <qapplication.h>
#include <qfontmetrics.h>
#include "pixmaploader.h"

#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "konsole.h"
#include "keytrans.h"


static const char *commonCmds[] =
{
    //"ls",
    "ls -la ",
    "cd ",
    //"pwd",
    //"cat",
    "less ",
    "vi ",
    "man ",
    "echo ",
    "set ",
    //"ps",
    "ps -aux\r",
    //"tar",
    //"tar -zxf",
    "grep ",
    //"grep -i",
    //"mkdir",
    "cp ",
    "mv ",
    //"rm",
    //"rmdir",
    //"chmod",
    "su\r",
    "top\r",
    //"find",
    //"make",
    //"tail",

/*
    "gzip",
    "gunzip",
    "chgrp",
    "chown",
    "date",
    "dd",
    "df",
    "dmesg",
    "fuser",
    "hostname",
    "kill",
    "killall",
    "ln",
    "ping",
    "mount",
    "more",
    "sort",
    "touch",
    "umount",
    "mknod",
    "netstat",
*/

    "exit\r",
    NULL
};


Konsole::Konsole(QWidget* parent, const char* name, WFlags fl) :
    QMainWindow(parent, name, fl)
{
    QStrList args;
    init("/bin/sh",args);
}

Konsole::Konsole(const char* name, const char* _pgm, QStrList & _args, int)
 : QMainWindow(0, name)
{
    init(_pgm,_args);
}

void Konsole::closeEvent( QCloseEvent * )
{
    doneSession( se, 0 );
}

void Konsole::init(const char* _pgm, QStrList & _args)
{
  se = 0L;
  b_scroll = TRUE; // histon;
  n_keytab = 0;
  n_render = 0;

  QFont font( "Helvetica", 6, QFont::Normal );
  font.setFixedPitch( FALSE );
  QFont fixedFont( "Micro", 4, QFont::Normal );
  fixedFont.setFixedPitch( TRUE );

  setFont( font );

  // create terminal emulation framework ////////////////////////////////////
  te = new TEWidget(this);
  te->setMinimumSize(150,70);    // allow resizing, cause resize in TEWidget
  te->setBackgroundPixmap( PixmapLoader::loadPixmap("faded_bg") );
  te->setVTFont(fixedFont);

  // create terminal toolbar ////////////////////////////////////////////////
  QToolBar *menuToolBar = new QToolBar( this );
  QMenuBar *menuBar = new QMenuBar( menuToolBar );

  QPopupMenu *commonPopup = new QPopupMenu( this );
  for (int i = 0; commonCmds[i] != NULL; i++)
    commonPopup->insertItem( commonCmds[i], i );
  connect( commonPopup, SIGNAL( activated(int) ), this, SLOT( enterCommand(int) ));
  
  QPopupMenu *fontList = new QPopupMenu( this );
  fontList->insertItem( "Micro", 0 );
  fontList->insertItem( "Small Fixed", 1 );
  fontList->insertItem( "Medium Fixed", 2 );
  fontList->insertItem( "Small Helvetica", 3 );
  fontList->insertItem( "Large Helvetica", 4 );
  connect( fontList, SIGNAL( activated(int) ), this, SLOT( fontChanged(int) ));

  menuBar->insertItem( "Command", commonPopup );
  menuBar->insertItem( "Font", fontList );

  QToolBar *toolbar = new QToolBar( this );

  // Button Commands
  QPushButton *enterButton = new QPushButton( toolbar, "Enter" );
  enterButton->setPixmap( PixmapLoader::loadPixmap( "enter" ) );
//  enterButton->setBorder( 2 );
  connect( enterButton, SIGNAL( pressed() ), this, SLOT( hitEnter() ));
  QPushButton *spaceButton = new QPushButton( toolbar, "Space" );
  spaceButton->setPixmap( PixmapLoader::loadPixmap( "space" ) );
  connect( spaceButton, SIGNAL( pressed() ), this, SLOT( hitSpace() ));
  QPushButton *tabButton = new QPushButton( toolbar, "Tab" );
  tabButton->setPixmap( PixmapLoader::loadPixmap( "tab" ) );
  connect( tabButton, SIGNAL( pressed() ), this, SLOT( hitTab() ));
  QPushButton *pasteButton = new QPushButton( toolbar, "Paste" );
  pasteButton->setPixmap( PixmapLoader::loadPixmap( "paste" ) );
  connect( pasteButton, SIGNAL( pressed() ), this, SLOT( hitPaste() ));
/*
  QPushButton *upButton = new QPushButton( toolbar, "Up" );
  upButton->setText( "Up" );
  upButton->setFlat( TRUE );
  connect( upButton, SIGNAL( pressed() ), this, SLOT( hitUp() ));
  QPushButton *downButton = new QPushButton( toolbar, "Down" );
  downButton->setText( "Down" );
  downButton->setFlat( TRUE );
  connect( downButton, SIGNAL( pressed() ), this, SLOT( hitDown() ));
*/
  setToolBarsMovable ( FALSE );
  addToolBar( menuToolBar );
  addToolBar( toolbar );

  // create applications /////////////////////////////////////////////////////
  setCentralWidget(te);

  // load keymaps ////////////////////////////////////////////////////////////
  KeyTrans::loadAll();
  for (int i = 0; i < KeyTrans::count(); i++)
  { KeyTrans* s = KeyTrans::find(i);
    assert( s );
  }

  // construct initial session ///////////////////////////////////////////////
  se = new TESession(this,te,_pgm,_args,"xterm");
  connect( se, SIGNAL(done(TESession*,int)), this, SLOT(doneSession(TESession*,int)) );
  se->setHistory(b_scroll);

  // read and apply default values ///////////////////////////////////////////
  resize(321, 321); // Dummy.
  QSize currentSize = size();
  if (currentSize != size())
     defaultSize = size();

  // activate and run first session //////////////////////////////////////////
  runSession(se);
}

Konsole::~Konsole()
{
    if (te)
	delete te;
}

void Konsole::fontChanged(int f)
{
    QFont fixedFont;

    switch (f)
    {
        case 0:
            fixedFont = QFont( "Micro", 4, QFont::Normal );
            break;
        case 1:
            fixedFont = QFont( "Fixed", 7, QFont::Normal );
            break;
        case 2:
            fixedFont = QFont( "Fixed", 12, QFont::Normal );
            break;
        case 3:
            fixedFont = QFont( "Helvetica", 6, QFont::Normal );
            break;
        case 4:
            fixedFont = QFont( "Helvetica", 10, QFont::Normal );
            break;
    }

    fixedFont.setFixedPitch( (f < 3) ? TRUE : FALSE );
    te->setVTFont(fixedFont);
}

void Konsole::enterCommand(int c)
{
    QString text = commonCmds[c];
    te->emitText(text);
}

void Konsole::hitEnter()
{
    te->emitText(QString("\r"));
}

void Konsole::hitSpace()
{
    te->emitText(QString(" "));
}

void Konsole::hitTab()
{
    te->emitText(QString("\t"));
}

void Konsole::hitPaste()
{
    te->pasteClipboard();
}

void Konsole::hitUp()
{
//    te->emitKey();
}

void Konsole::hitDown()
{
//    te->emitKey();
}

/**
   This function calculates the size of the external widget
   needed for the internal widget to be
 */
QSize Konsole::calcSize(int columns, int lines) {
    QSize size = te->calcSize(columns, lines);
    return size; 
}

/**
    sets application window to a size based on columns X lines of the te
    guest widget. Call with (0,0) for setting default size.
*/

void Konsole::setColLin(int columns, int lines)
{
  if ((columns==0) || (lines==0))
  {
    if (defaultSize.isEmpty()) // not in config file : set default value
    {
      defaultSize = calcSize(80,24);
      // notifySize(24,80); // set menu items (strange arg order !)
    }
    resize(defaultSize);
  } else {
    resize(calcSize(columns, lines));
    // notifySize(lines,columns); // set menu items (strange arg order !)
  }
}

/*
void Konsole::setFont(int fontno)
{
  QFont f;
  if (fontno == 0)
    f = defaultFont = QFont( "Helvetica", 12  );
  else
  if (fonts[fontno][0] == '-')
    f.setRawName( fonts[fontno] );
  else
  {
    f.setFamily(fonts[fontno]);
    f.setRawMode( TRUE );
  }
  if ( !f.exactMatch() && fontno != 0)
  {
    QString msg = i18n("Font `%1' not found.\nCheck README.linux.console for help.").arg(fonts[fontno]);
    QMessageBox(this,  msg);
    return;
  }
  if (se) se->setFontNo(fontno);
  te->setVTFont(f);
  n_font = fontno;
}
*/

// --| color selection |-------------------------------------------------------

void Konsole::changeColumns(int columns)
{
  setColLin(columns,te->Lines());
  te->update();
}

void Konsole::runSession(TESession* s)
{
    activateSession(s);
  // give some time to get through the
  // resize events before starting up.
    QTimer::singleShot(100,s,SLOT(run()));
}

void Konsole::activateSession(TESession *s)
{
  se = s;
  te->currentSession = se;
  s->setConnect(TRUE);
}


//FIXME: If a child dies during session swap,
//       this routine might be called before
//       session swap is completed.

void Konsole::doneSession(TESession* s, int )
{
  s->setConnect(FALSE);
  delete s;
  delete this;
}


