/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <stdlib.h>

#include <qimage.h>
#include <qtextstream.h>
#include <qtextcodec.h>
#include <qxml.h>
#include <qfileinfo.h>

#include <xmlreader.h>
#include <pixmaploader.h>

#include "filetypeinfodb.h"

FileTypeInfoDB::FileTypeInfoDB()
{

/*  
    setType( "rpm","application/rpm", "rpmfile.xpm" );
    setType( "tgz", "application/tgz", "tgz.xpm" );
    setType( "gz", "application/gz", "tgz.xpm" );
    setType( "wav", "application/wav", "kcmsyssound.xpm" );
    setType( "pdf", "application/pdf", "acroread.xpm" );
    setType( "cpp", "text/cpp", "c_src.xpm" );
    setType( "h", "text/h", "h_src.xpm" );
    setType( "c", "text/c", "c_src.xpm");
    setType( "txt","text/plain", "txt.xpm" );
    setType( "xpm","image/xpm", "pixmap.xpm" );
*/

    QString fname =  getenv( "HOME" );
    fname += "/.filebrowserdb";

    if( !QFile( fname ).exists() ){
	fname = getenv( "QTDIR" );
	fname += "/etc/filebrowserdb";
    }
    
    load( fname );
}

void FileTypeInfoDB::load( const QString & fname )
{
    QFile f( fname );
    
    if ( !f.open( IO_ReadOnly ) )
        return;

    QTextStream ts( &f );
    QXmlInputSource inputsource( ts );
    QXmlSimpleReader reader;
    XmlHandler handler;

    reader.setFeature( "http://trolltech.com/xml/features/report-"
		       "whitespace-only-CharData", FALSE );
    reader.setContentHandler( &handler );
    reader.parse( inputsource );

    Node * n = handler.firstNode();
    
    while( n && n->tagName() != "FILEBROWSER" ) n = n->firstChild();
    while( n ){
	if( n->tagName() == "FILEBROWSER" ) n = n->firstChild();
	if( n->tagName() == "entry" ){
	    QString key, type, pixname, appname;
	    Node * n2 = n->firstChild();

	    if( n2->tagName() == "key" ){
		key = n2->data();
                n2 = n2->nextNode();
	    } 
	    if( n2->tagName() == "type" ){
		type = n2->data();
		n2 = n2->nextNode();
	    }
	    if( n2->tagName() == "pixmap" ){
		pixname = n2->data();
		n2 = n2->nextNode();
	    }
	    if( n2->tagName() == "appname" ){
		appname = n2->data();
	    }
	    setType( key, type, pixname, appname );
	}
	n = n->nextNode();
    }
}

void FileTypeInfoDB::save( const QString & fname )
{
    QFile f( fname );
    
    if( !f.open( IO_WriteOnly ) )
	return;
    
    QTextStream ts( &f );
    ts.setCodec( QTextCodec::codecForName( "UTF-8" ) );
    ts << "<!DOCTYPE FILEBROWSER><FILEBROWSER>" << endl;
    
    FileTypes::Iterator it;
    for( it = db.begin(); it != db.end(); ++it ){
	ts << "<entry>" << endl;
        ts << "    " << "<key>" << it.key() << "</key>" << endl;
        ts << "    " << "<type>" << it.data().ftype << "</type>" << endl;
        ts << "    " << "<pixmap>" << it.data().pixname << "</pixmap>" << endl;
        ts << "    " << "<appname>" << it.data().appname << "</appname>" 
	   << endl;
	ts << "</entry>" << endl;
    }

    ts << "</FILEBROWSER>" << endl;        
}

void FileTypeInfoDB::setType( const QString & ext, const QString & type,
			      const QString & pix, 
			      const QString & appname )
{
    FileTypeInfo p;
    p.ftype   = type;
    p.pixname = pix;
    p.appname = appname;
    
    db.replace( ext, p );
}

QString FileTypeInfoDB::pixmap( const QString & s )
{
    // Try to find a key match..
    if( db.contains( s ) ){
	return db[ s ].pixname;
    }
    
    // .. or a type match
    QString t = type( s );
    FileTypes::Iterator it;
    for( it = db.begin(); it != db.end(); ++it ){
	if( it.data().ftype == t ){
	    return it.data().pixname;
	}
    }
    return "binary.xpm";
}

QString FileTypeInfoDB::appname( const QString & s )
{
    // Try to find a key match..
    if( db.contains( s ) ){
	return db[ s ].appname;
    }
    
    // .. or a type match
    QString t = type( s );
    FileTypes::Iterator it;
    for( it = db.begin(); it != db.end(); ++it ){
	if( it.data().ftype == t ){
	    return it.data().appname;
	}
    }
    return "";
}

QString FileTypeInfoDB::type( const QString & file )
{
    QFileInfo fi( file );
    
    QString type = "application/octet-stream";
    QString e    = fi.extension( FALSE );
    const char * imgfmt;
	
    if( db.contains( fi.fileName() ) )
	type = db[ fi.fileName() ].ftype.latin1();
    else if( db.contains( e ) )
	type = db[e].ftype.latin1();
    else if( fi.isReadable() && ( imgfmt = QImage::imageFormat( file ) ) )
	type = "image/" + QString( imgfmt ).lower();

    return type;
}

FileTypeInfoDB * FileTypeInfoDB::instance()
{
    static FileTypeInfoDB * _instance = NULL;
    
    if( _instance == NULL ){
	_instance = new FileTypeInfoDB();
    }
    
    return _instance;
}
