/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "filemanager.h"

#include <qfile.h>
#include <qdir.h>

#include <stdlib.h>

FileManager::FileManager()
{
}

QValueList<FileManager::File> FileManager::files( const QString &category, const QString &filter )
{
    QString s = QString( getenv( "HOME" ) ) + "/Documents/" + category;
    QDir d( s );
    d.setNameFilter( filter );
    QStringList lst = d.entryList();
    QValueList<File> res;
    for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
	File f;
	f.filename = *it;
	f.category = category;
	res << f;
    }
    return res;
}

QString FileManager::absoluteFileName( const File &f ) const
{
    return QString( getenv( "HOME" ) ) + "/Documents/" + f.category + "/" + f.filename;
}

bool FileManager::saveFile( const File &f, const QByteArray &data )
{
    QString fn = absoluteFileName(f);
    QFile fl( fn );
    if ( !fl.open( IO_WriteOnly ) )
	return FALSE;
    fl.writeBlock( data );
    fl.close();
    return TRUE;
}

bool FileManager::loadFile( const File &f, QByteArray &ba )
{
    QString fn = absoluteFileName(f);
    QFile fl( fn );
    if ( !fl.open( IO_ReadOnly ) )
	return FALSE;
    ba.resize( fl.size() );
    if ( fl.size() > 0 )
	fl.readBlock( ba.data(), fl.size() );
    fl.close();
    return TRUE;
}

bool FileManager::exists( const File &f )
{
    QValueList<File> fs = files( f.category, "*" );
    if ( fs.contains( f ) )
	return TRUE;
    return FALSE;
}
