/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "fileselector.h"
#include "global.h"
#include "pixmaploader.h"

#include <qdir.h>
#include <qwidget.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qtooltip.h>

FileSelectorItem::FileSelectorItem( QListView *parent, const FileManager::File &f )
    : QListViewItem( parent ), fl( f )
{
    setText( 0, QFileInfo( f.filename ).baseName() );
    setPixmap( 0, PixmapLoader::loadPixmap( "new" ) );
}


FileSelectorView::FileSelectorView( const QString &f, QWidget *parent, const char *name )
    : QListView( parent, name ), filter( f )
{
    setAllColumnsShowFocus( TRUE );
    addColumn( tr( "Name" ) );
    ///    addColumn( tr( "Type" ) );
//     addColumn( tr( "Size" ) ); ### todo
//     addColumn( tr( "Last Modified" ) );
    header()->hide();

    fileManager = new FileManager;
}

void FileSelectorView::showCategory( const QString &c )
{
    clear();
    QValueList<FileManager::File> files = fileManager->files( c, filter );
    for ( QValueList<FileManager::File>::Iterator it = files.begin(); it != files.end(); ++it )
	(void)new FileSelectorItem( this, *it );
}

FileSelector::FileSelector( const QString &f, QWidget *parent, const char *name )
    : QVBox( parent, name ), filter( f )
{
    setMargin( 3 );
    setSpacing( 2 );
    QHBox *top = new QHBox( this );
    top->setSpacing( 2 );

    categoryButton = new QPushButton( top );
    categories = new QPopupMenu( categoryButton );
    categoryButton->setPopup( categories );
    setupCategoryMenu();
    connect( categories, SIGNAL( activated( int ) ),
	     this, SLOT( categorySelected( int ) ) );

    (void)new QWidget( top );

    QToolButton *tb = new QToolButton( top );
    tb->setPixmap( PixmapLoader::loadPixmap( "new" ) );
    connect( tb, SIGNAL( clicked() ), this, SLOT( createNew() ) );
    buttonNew = tb;
    tb->setFixedSize( 18, 20 ); // tb->sizeHint() );
    tb->setAutoRaise( TRUE );
    QToolTip::add( tb, tr( "Create a new Document" ) );
    
    tb = new QToolButton( top );
    tb->setPixmap( PixmapLoader::loadPixmap( "close" ) );
    connect( tb, SIGNAL( clicked() ), this, SIGNAL( closeMe() ) );
    buttonClose = tb;
    tb->setFixedSize( 18, 20 ); // tb->sizeHint() );
    tb->setAutoRaise( TRUE );
    QToolTip::add( tb, tr( "Close the File Selector" ) );

    view = new FileSelectorView( filter, this, "fileview" );
    categorySelected( 0 );
    connect( view, SIGNAL( clicked( QListViewItem * ) ),
	     this, SLOT( fileClicked( QListViewItem * ) ) );
}

void FileSelector::createNew()
{
    FileManager::File f;
    f.filename = "Unnamed";
    f.category = categoryButton->text();
    emit newSelected( f );
    emit closeMe();
}

void FileSelector::fileClicked( QListViewItem *i )
{
    if ( !i )
	return;
    emit fileSelected( ( (FileSelectorItem*)i )->file() );
    emit closeMe();
}

void FileSelector::setNewVisible( bool b )
{
    if ( b )
	buttonNew->show();
    else
	buttonNew->hide();
}

void FileSelector::setCloseVisible( bool b )
{
    if ( b )
	buttonClose->show();
    else
	buttonClose->hide();
}

void FileSelector::setupCategoryMenu()
{
    categories->clear();
    QStringList lst = Global::categories();
    int i = 0;
    for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it, ++i )
	categories->insertItem( *it, i, -1 );
/* ### Crashes
    categories->insertSeparator();
    categories->insertItem( tr( "Edit Categories..." ), this, SLOT( editCategories() ) );
*/
}

void FileSelector::categorySelected( int c )
{
    QStringList lst = Global::categories();
    view->showCategory( *lst.at( c ) );
    categoryButton->setText( *lst.at( c ) );
}

void FileSelector::editCategories()
{
}

void FileSelector::reread()
{
    view->showCategory( categoryButton->text() );
}
