/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef FILESELECTOR_H
#define FILESELECTOR_H

#include <qhbox.h>
#include <qvbox.h>
#include <qlistview.h>
#include <qtoolbutton.h>

#include "filemanager.h"

class QPopupMenu;
class QPushButton;

class FileSelectorItem : public QListViewItem
{
public:
    FileSelectorItem( QListView *parent, const FileManager::File &f );

    FileManager::File file() const { return fl; }

private:
    FileManager::File fl;

};

class FileSelectorView : public QListView
{
    Q_OBJECT

public:
    FileSelectorView( const QString &f, QWidget *parent, const char *name );
    void showCategory( const QString &c );

private:
    QString filter;
    FileManager *fileManager;

};

class FileSelector : public QVBox
{
    Q_OBJECT

public:
    FileSelector( const QString &f, QWidget *parent, const char *name );

    void setNewVisible( bool b );
    void setCloseVisible( bool b );
    void reread();

signals:
    void fileSelected( const FileManager::File & );
    void newSelected( const FileManager::File & );
    void closeMe();

private slots:
    void createNew();
    void fileClicked( QListViewItem * );
    void categorySelected( int );
    void editCategories();

private:
    void setupCategoryMenu();

private:
    FileSelectorView *view;
    QPopupMenu *categories;
    QPushButton *categoryButton;
    QString filter;
    QToolButton *buttonNew, *buttonClose;

};

#endif
