/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef MPEGVIEW_H
#define MPEGVIEW_H

#include <qwidget.h>
#include <qdatetime.h>

typedef struct mpeg3_rec mpeg3_t;

class MpegView : public QWidget {
    Q_OBJECT
public:
    MpegView( QWidget* parent=0, const char* name=0, WFlags fl=0 );
    ~MpegView();

    long currentFrame() const { return current_frame; }
    long totalFrames() const;

    QSize sizeHint() const;
    bool looping() const { return isLooping; }
    int volume() const;
    
public slots:
    void setCurrentFrame(long);
    void setCurrentFrame(int);

    bool play(const QString& filename);
    void playAgain();
    void stop();
    void setLooping( bool );
    void setVolume( int );
    void setMute( bool );
    
signals:
    void clicked();
    void frameDone(long);

protected:
    void timerEvent(QTimerEvent*);
    void mouseReleaseEvent(QMouseEvent* event);
private:
    void openAudio();
    bool playAudio();
    QTime playtime;
    int audioPauseTime;
    int audioSampleCounter;
    mpeg3_t *mpeg;
    long current_frame;
    long total_video_frames;
    long total_audio_samples;
    
    float framerate;
    int frameskip;
    int freq;
    int timerid;
    int stream;
    int framecount;
    int astreams;
    int channels;
    int vstreams;
    bool isLooping;
    bool isMuted;
    bool dontSkip;
    QString fileName;
};

#endif
