/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef __CANVAS_SHAPES_H__
#define __CANVAS_SHAPES_H__


#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qapplication.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qimage.h>
#include <qpaintdevice.h>


class CanvasRoundRect : QCanvasRectangle
{
public:
    CanvasRoundRect(int x, int y, QCanvas *canvas) :
	QCanvasRectangle( x, y, 23, 36, canvas) { setZ(0); show(); }
    void redraw(void) { hide(); show(); }	
protected:
    virtual void drawShape(QPainter &p) {
	p.drawRoundRect( x(), y(), 23, 36);
    }
};


class CanvasCircleOrCross : QCanvasRectangle
{
public:
    CanvasCircleOrCross(int x, int y, QCanvas *canvas) :
	QCanvasRectangle( x, y, 21, 21, canvas), circleShape(TRUE) { show(); }
    void redraw(void) { hide(); show(); }	
    void setCircle(void) { circleShape = TRUE; redraw(); }
    void setCross(void) { circleShape = FALSE; redraw(); }
protected:
    virtual void drawShape(QPainter &p) {
	// Green circle
	if (circleShape == TRUE) {
    	    p.setPen( QPen( QColor(0x10, 0xE0, 0x10), 1 ) );
    	    p.drawEllipse( x() - 1, y() - 1, 21, 21);
    	    p.drawEllipse( x() - 1, y() - 0, 21, 19);
    	    p.drawEllipse( x() + 0, y() + 0, 19, 19);
    	    p.drawEllipse( x() + 1, y() + 0, 17, 19);
    	    p.drawEllipse( x() + 1, y() + 1, 17, 17);
	// Red cross
	} else {
    	    p.setPen( QPen( QColor(0xE0, 0x10, 0x10), 5 ) );
    	    p.drawLine( x(), y(), x() + 20, y() + 20);
    	    p.drawLine( x() + 20, y(), x(), y() + 20);
	}
    }
private:
    bool circleShape;
};


#endif

