/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <unistd.h>
#include <qdatetime.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qkeycode.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qimage.h>
#include <qprogressdialog.h>
#include <qpaintdevice.h>
#include <stdlib.h>
#include <time.h>
#include "solitaire.h"
#include "cardgame.h"
#include "canvasshapes.h"
#include "canvascard.h"


int highestZ = 0;
int numberOfFreeCells = 4;


class CanvasCardGame : public QCanvasView, public CardGame
{
public:
    CanvasCardGame(QCanvas &c, bool snap, QWidget *parent = 0, const char *name = 0, WFlags f = 0) :
    	QCanvasView( &c, parent, name, f ), 
	moved(FALSE), cardXOff(0), cardYOff(0),
	snapOn(snap),
	numberToDraw(1) { }

    virtual ~CanvasCardGame() {
	// the deletion stuff should be fixed now and only deletes
	// items created by this CardGame. I haven't verified there are zero
	// memory leaks yet
    }
    
    virtual Card *newCard( eValue v, eSuit s, bool f ) {
	return new CanvasCard( v, s, f, canvas() );
    }

    virtual void gameWon();
    virtual bool haveWeWon() { return FALSE; }

    virtual bool mousePressCard(Card *card, QPoint p) { Q_UNUSED(card); Q_UNUSED(p); return FALSE; }  
    virtual void mouseReleaseCard(Card *card, QPoint p) { Q_UNUSED(card); Q_UNUSED(p); }
//    virtual void mouseMoveCard(Card *card, QPoint p) { Q_UNUSED(card); Q_UNUSED(p); }
    void toggleSnap(void) { snapOn = (snapOn == TRUE) ? FALSE : TRUE; }
    void toggleCardsDrawn(void) { numberToDraw = (numberToDraw == 1) ? 3 : 1; }
    int cardsDrawn(void) { return numberToDraw; }
protected:
    void contentsMousePressEvent(QMouseEvent *e);
    void contentsMouseReleaseEvent(QMouseEvent *e);
    void contentsMouseMoveEvent(QMouseEvent *e);    
protected:
    // Mouse event state variables
    bool moved;
    CanvasCard *moving;
    int cardXOff, cardYOff;

/*
signals:
    void status(const QString&);
*/
private:
    bool snapOn;
    int numberToDraw;
};


void CanvasCardGame::gameWon() {
	
    srand(time(NULL));
    
    QCanvasItemList list = canvas()->allItems();
    QCanvasItemList::Iterator it = list.begin();
 
    for (; it != list.end(); ++it) {
	if ( (*it)->rtti() == canvasCardId ) {
	    // disperse the cards everywhere
	    int x = 300 - rand() % 1000;
	    int y = 300 + rand() % 200;
	    ((CanvasCard *)*it)->animatedMove( x, y, 50 );
	}
    }
/*
    QMessageBox *mb = new QMessageBox( "Play Again", 
	"<p>Would you like to play again?",
	QMessageBox::NoIcon, QMessageBox::Yes, QMessageBox::No, 0 );

    if ( mb->exec() == QMessageBox::Yes )
	newGame();
	
    delete mb;
*/
}


void CanvasCardGame::contentsMousePressEvent(QMouseEvent *e) {

    QCanvasItemList l = canvas()->collisions( e->pos() );
    
    for (QCanvasItemList::Iterator it = l.begin(); it != l.end(); ++it) {
	
	if ( (*it)->rtti() == canvasCardId ) {
	
	    moving = (CanvasCard *)*it;
	    
	    if ( moving->animated() )
		return;
		
	    cardXOff = (int)(e->pos().x() - moving->x());
	    cardYOff = (int)(e->pos().y() - moving->y());
	    
	    if ( !mousePressCard( moving, e->pos() ) ) {
		CanvasCard *card = moving;
		
		while (card) {
		    card->savePos();
		    card->setZ(highestZ);
		    highestZ++;
		    card = (CanvasCard *)card->getCardPile()->cardInfront(card);
		}
		
		moved = FALSE;
	    }
	    return;	
	}
    }
    
    mousePress( e->pos() );
    moving = NULL;
}

/*
//
// Should have some intelligent way to make double clicking on a
// card send it to the most appropriate pile
//
void CanvasCardGame::contentsMouseDoubleClickEvent(QMouseEvent *e) {
    QCanvasItemList l = canvas()->collisions( e->pos() );
    for (QCanvasItemList::Iterator it = l.begin(); it != l.end(); ++it) {
	if ( (*it)->rtti() == canvasCardId ) {
	    CanvasCard *card = (CanvasCard *)*it;

	    if ( card->animated() )
		return;
	    
	    if ( card->getCardPile()->isAllowedToBeMoved(card) ) {
		if (card->getCardPile()->cardInfront(card) == NULL) {
		    CardPile *pile = first();
		    if (pile && pile->isAllowedOnTop(card)) {
		       // move card to this pile
		       return;
		    }
		}
	    }
	}
    }
}
*/

void CanvasCardGame::contentsMouseMoveEvent(QMouseEvent *e) {
    
    QPoint p = e->pos();
    
    if ( moving ) {

	moved = TRUE;

	CanvasCard *item = moving;
	
	if (item->isFacing() != TRUE)
	    return;

	int tx = (int)p.x() - cardXOff; 
	int ty = (int)p.y() - cardYOff;

	if (snapOn == TRUE) {
			    
	    CardPile *pile = closestPile(tx, ty, 50);
	    
	    if (pile && pile->isAllowedOnTop(item)) {
		    
		    QPoint t = pile->getHypertheticalNextCardPos();
		    
		    int x = t.x(), y = t.y();
		    
		    CardPile *oldPile = item->getCardPile();
		    
		    if ( oldPile != pile) {
			while ( item ) {
			    item->move( x, y );
			    y += 13;
			    if (oldPile) {
				item = (CanvasCard *)oldPile->cardInfront(item);
			    } else {
				item = NULL;
			    }    
			}	
		    }
		    return;
	    }

	}
	
	while (item) {
	    item->move( tx, ty );
	    ty += 13;
	    item = (CanvasCard *)item->getCardPile()->cardInfront(item);
	}
    }
    
}


void CanvasCardGame::contentsMouseReleaseEvent(QMouseEvent *e)
{ 
    QPoint p = e->pos();

    Q_UNUSED(p);

    if ( moving ) {
	
	CanvasCard *item = moving;

	if ( item->animated() )
	    return;
	
	if ( moved ) {
	    
	    CardPile *pile = closestPile(item->x(), item->y(), 30);

	    if (pile && pile->isAllowedOnTop(item)) {
		    CardPile *oldPile = item->getCardPile();
		    Card *c = NULL;
		    if ( oldPile != pile) {
			while ( item ) {
			    if ( oldPile ) {
				c = oldPile->cardInfront(item);
				oldPile->removeCard(item);
			    }    
			    pile->addCardToTop(item);
			    item->setCardPile(pile);
			    item->move( pile->getCardPos(item) );

			    if (item->getValue() == king && haveWeWon()) {
				gameWon();
				return;
			    }
			    
			    if (oldPile) {
				item = (CanvasCard *)c;
			    } else {
				item = NULL;
			    }
			}	
			return;
		    }
	    }
	    
	    while (item) {
		item->animatedMove();
		item = (CanvasCard *)item->getCardPile()->cardInfront(item);
	    }
	}
    }

    moved = FALSE;
}


/*
// 
// Currently the card deck isn't implemented as a pile, it's a thing todo
//
class SolitaireFaceDownDeck : public CardPile, public CanvasRoundRect
{
public:
    SolitaireFaceDownDeck(int x, int y, QCanvas *canvas)
        : CardPile(x, y), CanvasRoundRect(x, y, canvas) { }
    virtual bool isAllowedOnTop(Card *card) {
        return FALSE;	
    }
    virtual bool isAllowedToBeMoved(Card *card) {
	if ( (!card->isFacing()) && ( card == cardOnTop() ) )
	    return TRUE;
        return FALSE;	
    }
};
*/


class SolitaireFaceDownDeck : public CardPile, public CanvasRoundRect
{
public:
    SolitaireFaceDownDeck(int x, int y, QCanvas *canvas)
        : CardPile(x, y), CanvasRoundRect(x, y, canvas) { }
    virtual bool isAllowedOnTop(Card *card) {
	Q_UNUSED(card);
        return FALSE;	
    }
    virtual bool isAllowedToBeMoved(Card *card) {
	if ( (!card->isFacing()) && ( card == cardOnTop() ) )
	    return TRUE;
        return FALSE;	
    }
};


class SolitaireDiscardPile : public CardPile, public CanvasRoundRect
{
public:
    SolitaireDiscardPile(int x, int y, QCanvas *canvas)
        : CardPile(x, y), CanvasRoundRect(x, y, canvas) { }
    virtual bool isAllowedOnTop(Card *card) {
	if ( card->isFacing() && ( card->getCardPile()->cardInfront(card) == NULL ) &&
	   ( ( ( cardOnTop() == NULL ) && ( card->getValue() == ace ) ) ||
	     ( ( cardOnTop() != NULL ) &&
	       ( (int)card->getValue() == (int)cardOnTop()->getValue() + 1 ) &&
	       ( card->getSuit() == cardOnTop()->getSuit() ) ) ) )
	    return TRUE;
        return FALSE;	
    }
    virtual bool isAllowedToBeMoved(Card *card) {
	if ( card->isFacing() && ( card == cardOnTop() ) )
	    return TRUE;
        return FALSE;	
    }
};


class SolitaireWorkingPile : public CardPile, public CanvasRoundRect
{
public:
    SolitaireWorkingPile(int x, int y, QCanvas *canvas)
        : CardPile(x, y), CanvasRoundRect(x, y, canvas), top(x, y) { }
    virtual bool isAllowedOnTop(Card *card) {
	if ( card->isFacing() &&
	     ( ( ( cardOnTop() == NULL ) && (card->getValue() == king) ) ||
	       ( ( cardOnTop() != NULL ) &&
 	         ( (int)card->getValue() + 1 == (int)cardOnTop()->getValue() ) &&
	         ( card->isRed() != cardOnTop()->isRed() ) ) ) )
	    return TRUE;
        return FALSE;	
    }
    virtual bool isAllowedToBeMoved(Card *card) {
	if ( card->isFacing() )
	    return TRUE;
        return FALSE;	
    }
    virtual void cardAddedToTop(Card *card) {
	Q_UNUSED(card);
	top = getCardPos(NULL);
	setNextX( top.x() );
	setNextY( top.y() );
    }
    virtual void cardRemoved(Card *card) {
	Q_UNUSED(card);

	Card *newTopCard = cardOnTop();

	if ( !newTopCard ) {
	    top = QPoint( pileX, pileY );
	    setNextX( pileX );
    	    setNextY( pileY );
	    return;
	} else {
	    top = getCardPos(NULL);
	    if ( newTopCard->isFacing() == FALSE ) {
		// correct the position taking in to account the card is not
    		// yet flipped, but will become flipped
    		top = QPoint( top.x() - 1, top.y() - 3 );
		newTopCard->flipTo( top.x(), top.y() );
		top = QPoint( top.x(), top.y() + 13 );
	    }
    	    setNextX( top.x() );
	    setNextY( top.y() );
	}
    }
    virtual QPoint getCardPos(Card *c) {
	int x = pileX, y = pileY;
	Card *card = cardOnBottom();
	while ((card != c) && (card != NULL)) {
	    if (card->isFacing()) {
		y += 13;    
	    } else {
		x += 1;    
		y += 3;
	    }
	    card = cardInfront(card); 
	}
	return QPoint( x, y );
    }
    virtual QPoint getHypertheticalNextCardPos(void) {
	return top;
	// return QPoint( getNextX(), getNextY() );
    }
private:
    QPoint top;

};


class FreecellDiscardPile : public SolitaireDiscardPile
{
public:
    FreecellDiscardPile(int x, int y, QCanvas *canvas) :
    	SolitaireDiscardPile(x, y, canvas) { }

};


class FreecellWorkingPile : public SolitaireWorkingPile
{
public:
    FreecellWorkingPile(int x, int y, QCanvas *canvas) :
    	SolitaireWorkingPile(x, y, canvas) { }

    virtual bool isAllowedToBeMoved(Card *card) {
	int nextExpectedValue = (int)card->getValue();
	bool nextExpectedColor = card->isRed();
	int numberOfCardsBeingMoved = 0;
	
	while ((card != NULL)) {
	    numberOfCardsBeingMoved++;
	    if ( (int)card->getValue() != nextExpectedValue )
		return FALSE;
	    if ( card->isRed() != nextExpectedColor )
		return FALSE;
	    nextExpectedValue--;;
	    nextExpectedColor = !nextExpectedColor;
	    card = cardInfront(card); 
	}
       
	if (numberOfCardsBeingMoved <= (numberOfFreeCells + 1))
	    return TRUE;
	
	return FALSE;
    }
    virtual void cardRemoved(Card *card) {
	if ( !isDealing() &&  !cardOnTop() )
	    numberOfFreeCells++;
	SolitaireWorkingPile::cardRemoved( card );
    }
    virtual void cardAddedToTop(Card *card) {
	if ( !isDealing() && cardOnBottom() == card )
	    numberOfFreeCells--;
	SolitaireWorkingPile::cardAddedToTop( card );
    }
};


class FreecellFreecellPile : public CardPile, public CanvasRoundRect
{
public:
    FreecellFreecellPile(int x, int y, QCanvas *canvas)
        : CardPile(x, y), CanvasRoundRect(x, y, canvas) { }
    virtual bool isAllowedOnTop(Card *card) {
	if ( ( cardOnTop() == NULL ) && ( card->getCardPile()->cardInfront(card) == NULL  ) )
	    return TRUE;
        return FALSE;	
    }
    virtual bool isAllowedToBeMoved(Card *card) {
	Q_UNUSED(card);
    	return TRUE;
    }
    virtual void cardAddedToTop(Card *card) {
	Q_UNUSED(card);
	numberOfFreeCells--;
    }
    virtual void cardRemoved(Card *card) {
	Q_UNUSED(card);
	numberOfFreeCells++;
    }
};


class FreecellCardGame : public CanvasCardGame
{
public:
    FreecellCardGame(QCanvas *c, bool snap, QWidget *parent = 0);
    virtual void deal(void);
    virtual bool haveWeWon() { 
	return ( discardPiles[0]->kingOnTop() &&
		 discardPiles[1]->kingOnTop() &&
		 discardPiles[2]->kingOnTop() &&
		 discardPiles[3]->kingOnTop() );
    }
    virtual void mousePress(QPoint p) { Q_UNUSED(p); }
    virtual void mouseRelease(QPoint p) { Q_UNUSED(p); }
//    virtual void mouseMove(QPoint p);
    virtual bool mousePressCard(Card *card, QPoint p);
    virtual void mouseReleaseCard(Card *card, QPoint p) { Q_UNUSED(card); Q_UNUSED(p); }
//    virtual void mouseMoveCard(Card *card, QPoint p) { Q_UNUSED(card); Q_UNUSED(p); }
    bool snapOn;
private:
    FreecellWorkingPile *workingPiles[8];
    FreecellDiscardPile *discardPiles[4];
};


FreecellCardGame::FreecellCardGame(QCanvas *c, bool snap, QWidget *parent) : CanvasCardGame(*c, snap, parent)
{
    numberOfFreeCells = 4;

    for (int i = 0; i < 4; i++) {
	addCardPile( new FreecellFreecellPile( 5 + i * 28, 10, canvas() ) );
    }
    for (int i = 0; i < 4; i++) {
	discardPiles[i] = new FreecellDiscardPile( 125 + i * 28, 10, canvas() );
	addCardPile(discardPiles[i]);
    }
    for (int i = 0; i < 8; i++) {
	workingPiles[i] = new FreecellWorkingPile( 10 + i * 28, 50, canvas() );
	addCardPile(workingPiles[i]);
    }
}


void FreecellCardGame::deal(void)
{
    highestZ = 1;
    
    beginDealing();
    
    for (int i = 0; i < 52; i++) {
	Card *card = cards[i];
	card->setFace( TRUE );
	card->setPos( 0, 0, highestZ );
	card->setCardPile( workingPiles[i%8] );
	workingPiles[i%8]->addCardToTop( card );
	card->move( workingPiles[i%8]->getCardPos( card ) );
	card->showCard();
	highestZ++;
    }

    endDealing();
}


bool FreecellCardGame::mousePressCard( Card *c, QPoint p )
{
    Q_UNUSED(p);
    
    if ( !c->getCardPile()->isAllowedToBeMoved(c) ) {
	moving = NULL;
	return TRUE;
    }

    return FALSE;
}


class SolitaireCardGame : public CanvasCardGame
{
public:
    SolitaireCardGame(QCanvas *c, bool snap, QWidget *parent = 0);
    virtual ~SolitaireCardGame();
    virtual void deal(void);
    virtual bool haveWeWon() { 
	return ( discardPiles[0]->kingOnTop() &&
		 discardPiles[1]->kingOnTop() &&
		 discardPiles[2]->kingOnTop() &&
		 discardPiles[3]->kingOnTop() );;
    }
    virtual void mousePress(QPoint p);
    virtual void mouseRelease(QPoint p) { Q_UNUSED(p); }
//    virtual void mouseMove(QPoint p);
    virtual bool mousePressCard(Card *card, QPoint p);
    virtual void mouseReleaseCard(Card *card, QPoint p) { Q_UNUSED(card); Q_UNUSED(p); }
//    virtual void mouseMoveCard(Card *card, QPoint p) { Q_UNUSED(card); Q_UNUSED(p); }
    bool canTurnOverDeck(void) { return (numberOfTimesThroughDeck != 3); }
    void throughDeck(void) {
	numberOfTimesThroughDeck++;
	if (numberOfTimesThroughDeck == 3)
    	    circleCross->setCross();
    }
    bool snapOn;
private:
    CanvasCircleOrCross *circleCross;
    CanvasRoundRect *rectangle;
    SolitaireWorkingPile *workingPiles[7];
    SolitaireDiscardPile *discardPiles[4];
    int numberOfTimesThroughDeck;
};


SolitaireCardGame::SolitaireCardGame(QCanvas *c, bool snap, QWidget *parent) : CanvasCardGame(*c, snap, parent)
{
    circleCross = new CanvasCircleOrCross( 6, 16, canvas() );
    rectangle = new CanvasRoundRect(  35, 10, canvas() );
    numberOfTimesThroughDeck = 0;

    for (int i = 0; i < 4; i++) {
	discardPiles[i] = new SolitaireDiscardPile( 110 + i * 30, 10, canvas() );
	addCardPile(discardPiles[i]);
    }
    for (int i = 0; i < 7; i++) {
	workingPiles[i] = new SolitaireWorkingPile( 10 + i * 30, 50, canvas() );
	addCardPile(workingPiles[i]);
    }
}


SolitaireCardGame::~SolitaireCardGame()
{
    delete circleCross;
    delete rectangle;
}


void SolitaireCardGame::deal(void)
{
    highestZ = 1;
    int t = 0;

    beginDealing();
/* 
    int discardPileValues[4] = { 0, 0, 0, 0 };
    // cheat to speed up winning
    for ( ; t < 52; t++) {
	Card *card = cards[t];
	int p =  t % 4;
	card->setPos( 0, 0, highestZ );
	card->move( discardPiles[p]->getCardPos(card) );
	card->setFace(TRUE);
	card->showCard();
	discardPiles[p]->addCardToTop(card);
	card->setCardPile(discardPiles[p]);
	discardPileValues[p]++;
	t++;
	card = cards[t];
	highestZ++;
	card->setFace(TRUE);
    }
    gameWon();
    return;
*/
    for (int i = 0; i < 7; i++) {
    	int x = 10 + i * 31;
    	int y = 50 + i * 3;
	cards[t]->setFace(TRUE);
	for (int k = i; k < 7; x += 30, k++, t++) {
	    Card *card = cards[t];
	    card->setPos( x, y, highestZ );
	    card->showCard();
	    workingPiles[k]->addCardToTop(card);
	    card->setCardPile(workingPiles[k]);
	    highestZ++;
	}
    }
    
    for ( ; t < 52; t++) {
	Card *card = cards[t];
	card->setPos( 5, 10, highestZ );
	card->showCard();
	highestZ++;
    }

    endDealing();
}


bool SolitaireCardGame::mousePressCard( Card *card, QPoint p )
{
    Q_UNUSED(p);

    CanvasCard *item = (CanvasCard *)card;
    if (item->isFacing() != TRUE) {
	// From facedown stack
	if ((item->x() == 5) && (item->y() == 10)) {
	    item->setZ(highestZ);
	    highestZ++;
	    item->flipTo( 35, item->y() );
	}
	moving = NULL;
	moved = FALSE;

	// move two other cards if we flip three at a time
	int flipped = 1;
	QCanvasItemList l = canvas()->collisions( p );
	for (QCanvasItemList::Iterator it = l.begin(); (it != l.end()) && (flipped != cardsDrawn()); ++it) {
	    if ( (*it)->rtti() == canvasCardId ) {
		CanvasCard *item = (CanvasCard *)*it;
		if (item->animated())
		    continue;
		item->setZ(highestZ);
		highestZ++;
		flipped++;
		item->flipTo( 35, item->y(), 8 * flipped );
	    }
	}
	
	return TRUE;
    }

    return FALSE;
}


void SolitaireCardGame::mousePress(QPoint p)
{
    if ( canTurnOverDeck() && 
         (p.x() >  5) && (p.x() < 28) &&
	 (p.y() > 10) && (p.y() < 46) ) {

	QCanvasItemList l2 = canvas()->allItems();

	for (QCanvasItemList::Iterator it = l2.begin(); it != l2.end(); ++it) {
	
	    if ( (*it)->rtti() == canvasCardId ) {
	    
		CanvasCard *item = (CanvasCard *)(*it);
		
    		// From faceup stack
		if ((item->x() == 35) && (item->y() == 10)) {
		    item->setZ(highestZ);
		    item->setX(5);
    		    highestZ++;
		    item->setFace(FALSE);		
		}
    
	    }

	}
	
	throughDeck();
	
	moved = TRUE;
    }
}


void CanvasCardWindow::initSolitaire()
{
    // Create New Game 
    if ( cardGame )
	delete cardGame;
    cardGame = new SolitaireCardGame( &canvas, snapOn, this );
    gameType = 0;
    setCaption("Solitaire");
    setCentralWidget(cardGame);
    cardGame->newGame();
    setCardBacks();
}


void CanvasCardWindow::initFreecell()
{
    // Create New Game
    if ( cardGame ) 
	delete cardGame;
    cardGame = new FreecellCardGame( &canvas, snapOn, this );
    gameType = 1;
    setCaption("Freecell");
    setCentralWidget(cardGame);
    cardGame->newGame();
    setCardBacks();
}


void CanvasCardWindow::snapToggle()
{
    snapOn = !snapOn;
    cardGame->toggleSnap();
}


void CanvasCardWindow::drawnToggle()
{
    cardGame->toggleCardsDrawn();
}


void CanvasCardWindow::setCardBacks()
{
    QCanvasItemList l = canvas.allItems();

    for (QCanvasItemList::Iterator it = l.begin(); it != l.end(); ++it) {
	if ( (*it)->rtti() == canvasCardId )
	    ((CanvasCard *)(*it))->setCardBack( cardBack );
    }
}


void CanvasCardWindow::changeCardBacks()
{
    cardBack++;

    if (cardBack == 5)
	cardBack = 0;

    setCardBacks();
}


CanvasCardWindow::CanvasCardWindow(QWidget* parent, const char* name, WFlags f) :
    QMainWindow(parent, name, f), canvas(230, 260), snapOn(TRUE), cardBack(4), gameType(0),
    cardGame(NULL)
{
    // Create Playing Area for Games
    if ( QPixmap::defaultDepth() < 12 ) {
//      canvas.setBackgroundColor(QColor(0x51, 0x74, 0x6B));
//	canvas.setBackgroundColor(QColor(0x20, 0xb0, 0x50));
	canvas.setBackgroundColor(QColor(0x08, 0x98, 0x2D));
    } else {
        const QPixmap BackgroundPixmap( findPixmap( "table_pattern" ), 0, ThresholdDither );  
        canvas.setBackgroundPixmap(BackgroundPixmap);
    }

#if defined( QT_QWS_CASSIOPEIA )
    canvas.setAdvancePeriod(70);
#else
    canvas.setAdvancePeriod(30);
#endif

#ifdef _SOLITAIRE_USE_ACCELS_
    QMenuBar* menu = menuBar();

    QPopupMenu* file = new QPopupMenu;
    file->insertItem("&New Game of Solitaire", this, SLOT(initSolitaire()), CTRL+Key_F);
    file->insertItem("&New Game or Freecell", this, SLOT(initFreecell()), CTRL+Key_F);
    file->insertSeparator();
    file->insertItem("&Close", this, SLOT(close()), CTRL+Key_W);
    menu->insertItem("&File", file);
    
    menu->insertSeparator();

    QPopupMenu* settings = new QPopupMenu;
    settings->insertItem("&Change Card Backs", this, SLOT(changeCardBacks()), Key_F2);
    settings->insertItem("&Snap On/Off", this, SLOT(snapToggle()), Key_F3);
    menu->insertItem("&Settings",settings);

    menu->insertSeparator();

    QPopupMenu* help = new QPopupMenu;
    help->insertItem("&About", this, SLOT(help()), Key_F1);
    help->setItemChecked(dbf_id, TRUE);
    menu->insertItem("&Help",help);
#else
    QMenuBar* menu = menuBar();

    QPopupMenu* file = new QPopupMenu;
    file->insertItem("New Game of Solitaire", this, SLOT(initSolitaire()));
    file->insertItem("New Game or Freecell", this, SLOT(initFreecell()));
    file->insertSeparator();
    file->insertItem("Close", this, SLOT(close()));
    menu->insertItem("File", file);
    
    menu->insertSeparator();

    QPopupMenu* settings = new QPopupMenu;
    settings->insertItem("Change Card Backs", this, SLOT(changeCardBacks()));
    settings->insertItem("Snap On/Off", this, SLOT(snapToggle()));
    settings->insertItem("Change Cards Drawn", this, SLOT(drawnToggle()));
    menu->insertItem("Settings",settings);

    menu->insertSeparator();

    QPopupMenu* help = new QPopupMenu;
    help->insertItem("About", this, SLOT(about()));
    help->insertItem("Help", this, SLOT(help()));
    help->setItemChecked(dbf_id, TRUE);
    menu->insertItem("Help",help);
#endif

    initSolitaire();
}


CanvasCardWindow::~CanvasCardWindow()
{
    if (cardGame)
	delete cardGame;
}


void CanvasCardWindow::resizeEvent(QResizeEvent *e)
{
    QSize s = e->size();
    canvas.resize( s.width() - 4, s.height() - 29);
}


void CanvasCardWindow::about()
{
    static QMessageBox *about = NULL;
    
    about = new QMessageBox( "Qt Card Game Example",
	"<h3>About Qt Card Games</h3>"
	"<p>"
	    "These games were implemented easily using Qt.<br>"
	    "The canvas widget was used as the basis<br>"
	    "on which the playing area is drawn. It<br>"
	    "demonstrates a real world application and one<br>"
	    "poissble use of the canvas widget. There are<br>"
	    "many other ways to use it, and there are also<br>"
	    "other ways the games could have been<br>"
	    "implemented with Qt without the canvas widget. "
	    "<p>Qt is flexible and easy to use!"
	"<p>", QMessageBox::NoIcon, 1, 0, 0, this, 0, FALSE );

    about->setButtonText( 1, "Dismiss" );
    about->show();
}


void CanvasCardWindow::help()
{
    static QMessageBox *about = NULL;
    
    if (gameType == 0) {
	about = new QMessageBox( "Qt Solitaire",
	    "<h3>Solitaire</h3>"
	    "<p>"
		"The object of the game is to stack all the<br>"
		"cards on the top piles from the Aces to<br>"
		"the Kings in suit. The rules are that you<br>"
		"can only move a red card of the next lowest<br>"
		"values on top of black cards in the lower<br>"
		"playing area, or a black on a red in order.<br>"
		"You can get new cards from the face down<br>"
		"stack to help you move the cards."
		"<p>Good luck and enjoy playing Solitaire!"
	    "<p>", QMessageBox::NoIcon, 1, 0, 0, this, 0, FALSE );
    } else if (gameType == 1) {
	about = new QMessageBox( "Qt Freecell",
	    "<h3>Freecell</h3>"
	    "<p>"
		"The object of the game is to stack all the<br>"
		"cards on the top right 4 piles from the Aces<br>"
		"to the Kings in suit. The rules are that<br>"
		"you can only move red cards of the next<br>"
		"lowest values on top of black cards, or<br>"
		"or black ones on to red ones in order.<br>"
		"You can move any card to the remaining<br>"
		"vacant freecells to help you move the cards."
		"<p>Good luck and enjoy playing Freecell!"
	    "<p>", QMessageBox::NoIcon, 1, 0, 0, this, 0, FALSE );
    } else {
	CanvasCardWindow::about();
    }

    if (about) {
        about->setButtonText( 1, "Dismiss" );
        about->show();
    }
}


void CanvasCardWindow::aboutQt()
{
    QMessageBox::aboutQt( this, "Qt Solitaire Example" );
}

