/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "pimspreadsheetwindow.h"
#include "fileselector.h"
#include "spreadsheet.h"
#include "global.h"
#include "pixmaploader.h"
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qlineedit.h>
#include <qaction.h>
#include <qapplication.h>
#include <qtoolbutton.h>
#include <qfileinfo.h>


PIMSpreadsheetWindow::PIMSpreadsheetWindow ( QWidget * parent, const char * name, WFlags f )
    : QMainWindow(parent,name,f)
{
    spreadsheet = new SpreadsheetWidget( this );
    spreadsheet->spreadsheet->setEditor( spreadsheet->equationEdit );

    setToolBarsMovable( FALSE );
    QToolBar *bar = new QToolBar( this );
    bar->setHorizontalStretchable( TRUE );
    menu = bar;

    QMenuBar *mb = new QMenuBar( bar );

    QPopupMenu *file = new QPopupMenu( this );
    QPopupMenu *edit = new QPopupMenu( this );

    bar = new QToolBar( this );
    editBar = bar;

    QAction *a = new QAction( tr( "New" ), PixmapLoader::loadPixmap( "new" ), QString::null, CTRL + Key_N, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileNew() ) );
    a->addTo( file );

    a = new QAction( tr( "Open" ), PixmapLoader::loadPixmap( "open" ), QString::null, CTRL + Key_O, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    a->addTo( file );

    a = new QAction( tr( "Close" ), PixmapLoader::loadPixmap( "close" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileClose() ) );
    a->addTo( file );

    a = new QAction( tr( "Equation" ), PixmapLoader::loadPixmap( "calculator" ), QString::null, CTRL + Key_E, this, 0 );
    connect( a, SIGNAL( activated() ), spreadsheet->spreadsheet, SLOT( editEquation() ) );
    a->addTo( bar );
    a->addTo( edit );

    a = new QAction( tr( "Clear Cell" ), PixmapLoader::loadPixmap( "cut" ), QString::null, CTRL + Key_K, this, 0 );
    connect( a, SIGNAL( activated() ), spreadsheet->spreadsheet, SLOT( clearCurrent() ) );
    a->addTo( bar );
    a->addTo( edit );

    mb->insertItem( tr( "&File" ), file );
    mb->insertItem( tr( "&Edit" ), edit );

    fileSelector = new FileSelector( "*.ss;*.ss", this, "fileselector" );
    fileSelector->setGeometry(0,0,width(),height());
    connect( fileSelector, SIGNAL( closeMe() ), this, SLOT( closeFileSelector() ) );
    connect( fileSelector, SIGNAL( newSelected( const FileManager::File &) ), this, SLOT( newFile( const FileManager::File & ) ) );
    connect( fileSelector, SIGNAL( fileSelected( const FileManager::File &) ), this, SLOT( openFile( const FileManager::File & ) ) );
    fileSelector->setCloseVisible( FALSE );
    setCentralWidget( spreadsheet );
    setCentralWidget( fileSelector );
    fileOpen();
    resize( 200, 300 );
}

PIMSpreadsheetWindow::~PIMSpreadsheetWindow()
{
    spreadsheet->spreadsheet->save();
}

void PIMSpreadsheetWindow::fileNew()
{
    fileSelector->reread();
    spreadsheet->spreadsheet->save();
    spreadsheet->spreadsheet->viewport()->setFocus();
    FileManager::File f;
    f.filename = "Unnamed";
    f.category = Global::categories().first();
    spreadsheet->spreadsheet->newFile( f );
}

void PIMSpreadsheetWindow::fileOpen()
{
    menu->hide();
    editBar->hide();
    spreadsheet->hide();
    spreadsheet->spreadsheet->hide();
    setCentralWidget( fileSelector );
    fileSelector->reread();
    fileSelector->raise();
    fileSelector->show();
}

void PIMSpreadsheetWindow::fileClose()
{
    spreadsheet->spreadsheet->save();
    fileOpen();
}

void PIMSpreadsheetWindow::newFile( const FileManager::File & f )
{
    fileSelector->hide();
    spreadsheet->spreadsheet->newFile( f );
}

void PIMSpreadsheetWindow::openFile( const FileManager::File & f )
{
    fileSelector->hide();
    spreadsheet->spreadsheet->openFile( f );
}

void PIMSpreadsheetWindow::closeFileSelector()
{
    menu->show();
    editBar->show();
    spreadsheet->show();
    setCentralWidget( spreadsheet );
    spreadsheet->spreadsheet->show();
    spreadsheet->raise();
}
