/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <qapplication.h>
#include <qtextstream.h>
#include <qdatetime.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qdir.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "common.h"
#include "wait.h"


#undef VERBOSE_DEBUG


bool getArg( QString &line, const QString &arg, QString **var )
{
    if ( line.startsWith( arg + "=" ) && var != NULL ) {
	*var = new QString( line.remove( 0, QString( arg ).length() + 1 ) );
	// printf("Found " + arg + ": %s\n", (const char *)**var);
	return TRUE;
    }
    return FALSE;
}


bool parseFileForArg( const QString &file, const QString &arg, QString **var )
{
    bool retVal = FALSE;
    QFile f( file );

    if ( f.exists() ) {
	f.open(IO_ReadOnly);
	QTextStream ts(&f);
	while (!ts.atEnd()) {
	    QString line = ts.readLine();
	    if ( getArg( line, arg, var ) ) retVal = TRUE;
	}
	f.close();
    }

    return retVal;
}


#define strToVar( var ) \
    getArg( line, #var, var )


void parseMenuFile( const QString &file, QString **Name, QString **Icon=NULL, QString **Exec=NULL, QString **Type=NULL, QString **Comment=NULL)
{
    QFile f( file );

    if ( f.exists() ) {

	f.open(IO_ReadOnly);
	QTextStream ts(&f);
//	QDict<char> dict( 17, FALSE );

	while (!ts.atEnd()) {
	    QString line = ts.readLine();

	    if ( line.startsWith("#") ) /* do nothing if it's a comment */ ;
	    else if ( line.startsWith("[") ) /* do nothing if it's a section */ ;
	    else if ( strToVar( Name ) ) ;
	    else if ( strToVar( Icon ) ) ;
	    else if ( strToVar( Exec ) ) ;
	    else if ( strToVar( Type ) ) ;
	    else strToVar( Comment );
	}

	f.close();
    }
}


void parseOptionsFile( const QString &file, QString **UseWidePopupMenu, QString **PopupMenuSidePixmap=NULL, QString **StartButtonIsFlat=NULL, QString **StartButtonPixmap=NULL, QString **UseMRUList=NULL)
{
    QFile f( file );

    if ( f.exists() ) {

	f.open(IO_ReadOnly);
	QTextStream ts(&f);

	while (!ts.atEnd()) {
	    QString line = ts.readLine();
	    if ( line.startsWith("#") ) /* do nothing */ ;
	    else if ( line.startsWith("[") ) ;
	    else if ( strToVar( UseWidePopupMenu ) ) ;
	    else if ( strToVar( PopupMenuSidePixmap ) ) ;
	    else if ( strToVar( StartButtonIsFlat ) ) ;
	    else if ( strToVar( StartButtonPixmap ) ) ;
	    else strToVar( UseMRUList );
	}

	f.close();
    }
}


#define ARRAY_SIZE(x)	    ( (int) ( sizeof((x)) / sizeof((x)[0]) ) )


#define FACTORY(T) \
    static void new##T( bool maximized ) { \
	QWidget *w = new T( 0, "test", QWidget::WDestructiveClose | QWidget::WGroupLeader ); \
	if ( maximized ) \
	    w->showMaximized(); \
	w->show(); \
    }


#if defined(QT_QWS_IPAQ) || defined(QT_QWS_CASSIOPEIA)
#include "../calibrate/calibrate.h"
#endif


#ifdef SINGLE_APP

#include "../addressbook/addressbook.h"
#include "../datebook/datebook.h"
#include "../filebrowser/filebrowser.h"
#include "../helpbrowser/helpbrowser.h"
#include "../minesweep/minesweep.h"
#include "../textedit/textedit.h"
#include "../todo/mainwindow.h"
#include "../clock/clock.h"
#include "../canvas/canvas.h"
#include "../calculator/calculatorimpl.h"
#include "../tux/tux.h"
#include "../tetrix/qtetrix.h"
#include "../sysinfo/sysinfo.h"
#if !defined(QT_DEMO_SINGLE_FLOPPY)
#if !defined(QT_QWS_CASSIOPEIA)
#include "../solitaire/solitaire.h"
#include "../embeddedkonsole/konsole.h"
#include "../spreadsheet/pimspreadsheetwindow.h"
#include "../wordgame/wordgame.h"
#include "../mpegplayer/mpegplayer.h"
#endif
#include "../settings/settings.h"
#endif

FACTORY(AddressbookWindow)
FACTORY(CalculatorImpl)
FACTORY(Clock)
FACTORY(CanvasWindow)
FACTORY(DateBook)
FACTORY(FileBrowser)
FACTORY(HelpBrowser)
FACTORY(MineSweep)
FACTORY(QTetrix)
FACTORY(TextEdit)
FACTORY(TodoWindow)
FACTORY(Tux)
FACTORY(SystemInfo)
#if !defined(QT_DEMO_SINGLE_FLOPPY)
#if !defined(QT_QWS_CASSIOPEIA)
FACTORY(CanvasCardWindow)
FACTORY(Konsole)
FACTORY(WordGame)
FACTORY(PIMSpreadsheetWindow)
FACTORY(MpegPlayer)
#endif
FACTORY(Settings)
#endif

#endif // SINGLE_APP


#if defined(QT_QWS_IPAQ) || defined(QT_QWS_CASSIOPEIA)
static void calibrate( bool )
{
    Calibrate *c = new Calibrate;
    c->show();
}
#endif


static void restart( bool )
{
    // Get the rotation configuration from the config file
    // ### this is a bit hacky and should be replaced with QCOP later
    QFile f(".rotation.conf");

    if (f.open(IO_ReadOnly)) {
	QString rs;
	QTextStream ts(&f);
	ts >> rs;
	setenv("QWS_DISPLAY", rs.latin1() , 1);
	f.close();
    }

    for ( int fd = 3; fd < 100; fd++ )
	close( fd );
#if defined(QT_DEMO_SINGLE_FLOPPY)
    execl( "/sbin/init", "qpe", 0 );
#elif defined(QT_QWS_CASSIOPEIA)
    execl( "/bin/sh", "sh", 0 );
#else
    execl( "./qpe", "qpe", 0 );
#endif
    exit(1);
}


static void quit( bool )
{
    qApp->quit();
}


struct Command {
    const char *file;
    void (*func)( bool );
    bool maximized;
} command[] = {

#ifdef SINGLE_APP

	{ "addressbook",	newAddressbookWindow,	 1 },
	{ "datebook",		newDateBook,		 1 },
	{ "filebrowser",	newFileBrowser,		 1 },
	{ "helpbrowser",	newHelpBrowser,		 1 },
	{ "textedit",		newTextEdit,		 1 },
	{ "todo",		newTodoWindow,		 1 },
	{ "calculator",		newCalculatorImpl,	 0 },
	{ "clock",		newClock,		 0 },
	{ "canvas",		newCanvasWindow,	 1 },
	{ "minesweep",		newMineSweep,		 1 },
	{ "tetrix",		newQTetrix,		 1 },
	{ "tux",		newTux,			 0 },
	{ "sysinfo",		newSystemInfo,		 0 },
#if !defined(QT_DEMO_SINGLE_FLOPPY)
#if !defined(QT_QWS_CASSIOPEIA)
	{ "spreadsheet",	newPIMSpreadsheetWindow, 1 },
	{ "embeddedkonsole",	newKonsole,		 1 },
	{ "wordgame",		newWordGame,		 1 },
	{ "mpegplayer",		newMpegPlayer,		 1 },
	{ "solitaire",		newCanvasCardWindow,	 1 },
#endif
	{ "settings",		newSettings,		 1 },
#endif


#endif // SINGLE_APP


#if defined(QT_QWS_IPAQ) || defined(QT_QWS_CASSIOPEIA)
        { "calibrate",		calibrate,		 1 },
#endif
#if !defined(QT_QWS_CASSIOPEIA)
        { "quit",		quit,			 1 },
#endif
        { "restart",		restart,		 1 },
};


#ifdef QT_NO_QWS_MULTIPROCESS
bool commandExists( const QString &name )
{
    for (int i = 0; i < ARRAY_SIZE(command); i++) {
	if ( QString(command[i].file) == name ) {
	    return TRUE;
	}
    }
    return FALSE;
}
#endif // QT_NO_QWS_MULTIPROCESS


bool executeClass( const QString &c )
{
#ifdef VERBOSE_DEBUG
    printf(" looking for class %s \n", c.latin1() );
#endif

    for (int i = 0; i < ARRAY_SIZE(command); i++) {
	if ( QString(command[i].file) == c ) {
#ifdef VERBOSE_DEBUG
	    printf(" found class %s \n", c.latin1() );
#endif
	    command[i].func( command[i].maximized );
#ifdef VERBOSE_DEBUG
	    printf(" started class %s \n", c.latin1() );
#endif
	    return TRUE;
	}
    }
#ifdef VERBOSE_DEBUG
    printf(" didn't find class %s \n", c.latin1() );
#endif
    return FALSE;
}


void executeFile( QString c )
{
#ifdef VERBOSE_DEBUG
    printf(" trying to execute %s \n", c.latin1() );
#endif

    // Display an hour-glass in the corner of the screen
    Wait *w = Wait::getWaitObject();

    if (w)
	w->setWaiting( TRUE );

    // attempt to execute the app using a builtin class for the app first
    // else try and find it in the bin directory
    if ( !executeClass( c ) )
#ifdef QT_NO_QWS_MULTIPROCESS
    {
	    QMessageBox::warning( 0, "Error", "Could not find the application " + c, "Ok", 0, 0, 0, 1 );
    }
#else    
     {
#ifdef VERBOSE_DEBUG
	printf(" looking for executable %s \n", c.latin1() );
#endif

	// Add our bin directory to the PATH environment
	static bool addedOurBinToPath = FALSE;

	if (!addedOurBinToPath) {
	    setenv( "PATH", (const char *)( QString( "../bin;") + getenv( "PATH" ) ), TRUE );
	    addedOurBinToPath = TRUE;
	}

/*
	//  ANSI C way of doing this
	printf(" About to execute %s\n", c.latin1());
	c += " 2>/dev/null >/dev/null &";
	system(c.latin1());
*/

	// Convert the command line in to a list of arguments
	QStringList list;
	unsigned int i1 = 0, len = 0, j;
	for ( j = 0; j < c.length(); j++, len++ )
	    if ( c[j].isSpace() )
		list += c.mid( i1, len ), i1 = j + 1, len = 0;
	list += c.mid( i1, len );
	const char **args = new (const char *)[list.count() + 1];
	for ( j = 0; j < list.count(); j++ ) {
	    args[j] = *(new QString( list[j].latin1() ));
#ifdef VERBOSE_DEBUG
	    printf("arg[%i]: %s\n", j, args[j]); // debug output
#endif
	}
	args[j] = NULL;

	if ( !QFile( args[0] ).exists() ) {
	    QMessageBox::warning( 0, "Error", "Could not find the executable " + c, "Ok", 0, 0, 0, 1 );
	} else if ( !::vfork() ) {
	    for ( int fd = 3; fd < 100; fd++ )
		::close( fd );
	    ::setpgid( ::getpid(), ::getppid() );
	    ::execv( args[0], (char * const *)args );
	    exit( -1 );
	}

    }
#endif //QT_NO_QWS_MULTIPROCESS    

    // Finished waiting
    // ### the hour-glass should go away when the app has started not here
    if (w)
	w->setWaiting( FALSE );
}


