/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <qmainwindow.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qdatetime.h>
#include <qfile.h>
#include <qdir.h>
#include <qmessagebox.h>
#include "qpeapplication.h"
#include "pixmaploader.h"
#include "startmenu.h"
#include "sidething.h"
#include "mrulist.h"
#include "common.h"


#undef TEST_AGAINST_KDE2


StartMenu::StartMenu(QWidget *parent) : QPushButton( parent ), mruList(NULL)
{
    loadOptions( QPEApplication::qpeDir() + "apps/.qpe_menu.conf");

    setPixmap( QPixmap( PixmapLoader::findPixmap( *startButtonPixmap ) ) );
    setFocusPolicy( NoFocus );
    setFlat( startButtonIsFlat );
    connect( this, SIGNAL(clicked()), this, SLOT(launch()) );

    loadMenus();
}


void StartMenu::fileTreePopupMenu( const QString &path, QPopupMenu *menu, int &item)
{
    QDir dirs( path, NULL, QDir::Name, QDir::Dirs );

    for ( unsigned int i = 0; i < dirs.count(); i++ ) {
	if ( ( dirs[i] != "." ) && ( dirs[i] != ".." ) ) {
	    QPopupMenu *new_menu = new QPopupMenu( menu );
	    menu->insertItem( QIconSet( QPixmap( PixmapLoader::findPixmap( "programs" ) ) ), dirs[i], new_menu );
	    loadDirMenus( path + "/" + dirs[i], new_menu, item );
	}
    }

    QDir files( path, NULL, QDir::Name, QDir::Files );

    for ( unsigned int i = 0; i < files.count(); i++ ) {
	AppLnk *appLnk = new AppLnk( files[i], "textedit_icon", files[i], "Document", "File", mruList );
	menu->insertItem( QIconSet( QPixmap( PixmapLoader::findPixmap( "textedit_icon" ) ) ), files[i], item );
	menu->connectItem( item, appLnk, SLOT(execute()) );
	item++;
    }
}


void StartMenu::loadDirMenus( const QString &path, QPopupMenu *menu, int &item)
{
    QDir dirs( QString( path + "/" ), NULL, QDir::Name, QDir::Dirs );

    for ( unsigned int i = 0; i < dirs.count(); i++ ) {
	if ( ( dirs[i] != "." ) && ( dirs[i] != ".." ) ) {
	    QString *name = new QString( dirs[i] );
	    QString *icon = new QString( "programs" );
	    if ( dirs.exists( dirs[i] + "/.directory" ) )
		parseMenuFile( QString( path + "/" + dirs[i] + "/.directory" ), &name, &icon );
	    // printf( " dir: %s \n", (const char *)dirs[i] );
	    QPopupMenu *new_menu = new QPopupMenu( menu );
	    if ( QFile( QPEApplication::qpeDir() + "pics/" + *icon + ".png" ).exists() )
		menu->insertItem( QIconSet( QPixmap( PixmapLoader::findPixmap( *icon ) ) ), *name, new_menu );
#ifdef TEST_AGAINST_KDE2
	    else /* a little hack so I can test this against the KDE2 applnk menu tree */
		menu->insertItem( QIconSet( QPixmap( "/opt/kde2/share/icons/locolor/16x16/apps/" + *icon + ".png" ) ), *name, new_menu );
#endif
	    loadDirMenus( QString( path + "/" + dirs[i] ), new_menu, item );
	    delete name;
	    delete icon;
	}
    }

    QDir files( path, NULL, QDir::Name, QDir::Files );

    for ( unsigned int i = 0; i < files.count(); i++ ) {
	QString *name = new QString( files[i] );
	QString *icon = new QString( "textedit_icon" );
	QString *exec = new QString( "textedit " + files[i] );
	QString *type = new QString( "File" );
	QString *comment = new QString( " " );
	if ( files[i].right( 8 ) == ".desktop" )
	    parseMenuFile( path + "/" + files[i], &name, &icon, &exec, &type, &comment );
	if ( *type == "Seperator" ) {
	    menu->insertSeparator();
#ifdef QT_NO_QWS_MULTIPROCESS
	} else if ( commandExists( *exec ) ) {
#else
	} else {
#endif
	    AppLnk *appLnk = new AppLnk( *name, *icon, *exec, *type, *comment, mruList );
	    if ( QFile( QPEApplication::qpeDir() + "pics/" + *icon + ".png" ).exists() )
		menu->insertItem( QIconSet( QPixmap( PixmapLoader::findPixmap( *icon ) ) ), *name, item );
#ifdef TEST_AGAINST_KDE2
	    else /* a little hack so I can test this against the KDE2 applnk menu tree */
		menu->insertItem( QIconSet( QPixmap( "/opt/kde2/share/icons/locolor/16x16/apps/" + *icon + ".png" ) ), *name, item );
#endif
	    menu->connectItem( item, appLnk, SLOT(execute()) );
	    item++;
	}
	delete name;
	delete icon;
	delete exec;
	delete type;
	delete comment;
    }
}


void StartMenu::loadOptions(const QString &configFile)
{
    // Read configuration file
    QString *tempBoolString1;
    QString *tempBoolString2;
    QString *tempBoolString3;
    parseOptionsFile( configFile, &tempBoolString1, &popupMenuSidePixmap, &tempBoolString2, &startButtonPixmap, &tempBoolString3 );
    useWidePopupMenu = ( *tempBoolString1 == "TRUE" ) ? TRUE : FALSE;
    startButtonIsFlat = ( *tempBoolString2 == "TRUE" ) ? TRUE : FALSE;
    useMRUList = ( *tempBoolString3 == "TRUE" ) ? TRUE : FALSE;
}


void StartMenu::loadMenus()
{
    int i = 0;

    if ( useWidePopupMenu )
	launchMenu = new PopupWithLaunchSideThing( this, popupMenuSidePixmap );
    else
        launchMenu = new QPopupMenu( this );

    QPopupMenu *menu = launchMenu;
//    menu->setMargin( 3 );
    
    if ( useMRUList )
        mruList = new MRUList( launchMenu, this );

    loadDirMenus( QPEApplication::qpeDir() + "apps", menu, i );

//  launchMenu->insertItem( QIconSet( QPixmap( PixmapLoader::findPixmap( "exec" ) ) ), "&Run...", this, SLOT(run()) );
//  launchMenu->insertItem( QIconSet( QPixmap( PixmapLoader::findPixmap( "quit_icon" ) ) ), "Sh&ut Down...", qApp, SLOT(quit()) );

    if ( useMRUList ) {
	launchMenu->insertSeparator();
        launchMenu->insertItem( mruList );
    }
}


void StartMenu::launch()
{
    int y = mapToGlobal( QPoint() ).y() - launchMenu->sizeHint().height();
    launchMenu->popup( QPoint( 1, y ) );
}


